using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace EveSkillNotify
{
    /// <summary>
    /// Holds information in a file
    /// </summary>
    public class Settings
    {
        /// <summary>
        /// Holds all settings in memory
        /// </summary>
        private SortedList<string, string> settings = new SortedList<string, string>();
        /// <summary>
        /// Stores the filename, default is settings.dat
        /// </summary>
        private string fileName = "settings.dat";
        /// <summary>
        /// Opens the file and loads all settings
        /// </summary>
        private void open()
        {
            try
            {
                //Check for existing file
                if (File.Exists(fileName))
                {
                    //open file for reading
                    TextReader file = new StreamReader(fileName);

                    string line = "";
                    //Read all lines
                    while ((line = file.ReadLine()) != null)
                    {
                        //Split the lines 
                        string[] lineSplit = line.Split(':');
                        //all colons in data is converted to &#58;
                        //convert back to normal
                        lineSplit[1].Replace("&#58;", ":");
                        //add setting to memory
                        settings.Add(lineSplit[0], lineSplit[1]);
                    }
                    //Close the file
                    file.Close();
                }
            }
            catch (UnauthorizedAccessException)
            {
                return;
            }
            catch
            {   //do nothing on error
            }

        }
        /// <summary>
        /// Saves the settings in memory to file
        /// </summary>
        private void save()
        {
            try
            {
                //open the file for writing
                TextWriter file = new StreamWriter(fileName, false);
                //Foreach setting
                foreach (KeyValuePair<string, string> setting in settings)
                {
                    //replace the colons in the data with &#58;
                    setting.Value.Replace(":", "&#58;");
                    //write the setting to the file
                    file.WriteLine(setting.Key + ":" + setting.Value);
                }
                //flush waiting writes
                file.Flush();
                //close the file
                file.Close();
            }
            catch (UnauthorizedAccessException)
            {
                return;
            }
            catch
            {   //do nothing on error
            }
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="inFileName">the file to use, empty string for default</param>
        public Settings(string inFileName)
        {
            //set the new filename if it isnt empty
            if (inFileName != "")
                fileName = inFileName;
            //load the settings
            open();
        }
        /// <summary>
        /// Saves the data to file,
        /// Dont know why this is used like this :o
        /// </summary>
        public void Save()
        {
            save();
        }
        /// <summary>
        /// Set/Modify a setting
        /// </summary>
        /// <param name="name">the setting name</param>
        /// <param name="value">the setting value</param>
        public void Set(string name, string value)
        {
            settings[name] = value;
            //save the new setting to file
            save();
        }
        /// <summary>
        /// Get the specified setting
        /// </summary>
        /// <param name="name">The setting to get</param>
        /// <returns>the value, empty for error</returns>
        public string Get(string name)
        {
            //if the setting exists
            if (settings.ContainsKey(name))
                //return it
                return settings[name];
            else
                return "";
        }
        /// <summary>
        /// Remove a setting
        /// </summary>
        /// <param name="name">the setting to remove</param>
        public void Remove(string name)
        {
            settings.Remove(name);
            save();
        }
        /// <summary>
        /// The number of settings on file
        /// </summary>
        /// <returns>The number of settings</returns>
        public int Count()
        {
            return settings.Count;
        }
        /// <summary>
        /// Check to see if a setting exists on file
        /// </summary>
        /// <param name="input">the setting to find</param>
        /// <returns></returns>
        public bool Contains(string input)
        {
            return settings.ContainsKey(input);
        }
    }
}