using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Media;
using System.Drawing;
using Microsoft.Win32;


namespace EveSkillNotify
{
    public class SkillNotify
    {
        private NotifyIcon trayicon = new NotifyIcon();
        private Timer notifyTimer = new Timer();

        private string EveUserID = "";
        private string EveApiKey = "";
        private bool NotifyWhenNoSkill = false;
        private bool NotifyWhenFinishedSkill = false;
        private bool NotifyWhenUnder5Mins = false;
        private bool NotifyWhenUnder1Hour = false;
        private bool NotifiedUpdateError = false;
        private int CheckInterval = 1;
        private bool ShowStatusOnTrayClick = false;
        private bool StartWithWindows = false;
        private bool ForceUpdateOnStart = false;

        private bool CharsLoaded = false;
        DateTime LastCheck = DateTime.Now;

        private bool Notified1HourWarning = false;
        private bool Notified5minWarning = false;
        private DateTime LastNoSkillNotify = DateTime.MinValue;
        private bool NotifiedSkillFinished = false;
        private int CurrentSkill = 0;

        private string WorkingDir = "";
        private EveApi api;
        private Settings EveSettings;
        private Settings SkillTypeIDs;

        private Main mainForm = null;

        public SkillNotify(Main inMainForm, string inWorkingDir)
        {
            WorkingDir = inWorkingDir;
            if (!System.IO.Directory.Exists(WorkingDir))
                WorkingDir = Environment.CurrentDirectory;

            SkillTypeIDs = new Settings(WorkingDir + @"\eveskills.txt");
            EveSettings = new Settings(WorkingDir + @"\settings.ini");
            api = new EveApi(SkillTypeIDs);

            if (inMainForm == null)
                mainForm = new Main(false,"");
            else
                mainForm = inMainForm;
            //Initialise tray icon
            trayicon.Text = "Eve Skill Notify";
            trayicon.Icon = new Icon(GetType(), "trayicon.ico");
            ContextMenu trayMenu = new ContextMenu();
            trayMenu.MenuItems.Add("About", new System.EventHandler(about_Click));
            trayMenu.MenuItems.Add("Settings", new System.EventHandler(Settings_Click));
            trayMenu.MenuItems.Add("Force Reload", new System.EventHandler(ForceReload_Click));
            trayMenu.MenuItems.Add("Show Skill Info", new System.EventHandler(ShowInfo_Click));
            trayMenu.MenuItems.Add("Exit", new System.EventHandler(Exit_Click));
            trayicon.ContextMenu = trayMenu;
            trayicon.DoubleClick += new EventHandler(trayicon_DoubleClick);
            trayicon.Visible = true;

            notifyTimer.Interval = 1000;
            notifyTimer.Tick += new EventHandler(notifyTimer_Tick);
            notifyTimer.Enabled = true;

            LoadSettings();

            if (ForceUpdateOnStart)
            {
                bool result = ReloadChars(false);
                if (result == false)
                {
                    if (NotifiedUpdateError == false)
                    {
                        NotifyWindow nw = new NotifyWindow();
                        nw.Text = "Cound Not Get Skill Information (" + api.ErrorText + ")";
                        nw.Title = "Retrieval Error";
                        nw.WaitTime = 10000;
                        nw.Notify();
                        WaitForRecheck = true;
                        SecondsUntilCheck = 20;
                        NotifiedUpdateError = true;
                    }
                }
                else
                {
                    WaitForRecheck = false;
                    SecondsUntilCheck = 0;
                    NotifiedUpdateError = false;
                }
            }
        }

        
        private void LoadSettings()
        {
            EveUserID = EveSettings.Get("UserID");
            EveApiKey = EveSettings.Get("ApiKey");
            NotifyWhenNoSkill = ConvertToBool(EveSettings.Get("NotifyWhenNoSkill"));
            NotifyWhenFinishedSkill = ConvertToBool(EveSettings.Get("NotifyWhenFinishedSkill"));
            NotifyWhenUnder5Mins = ConvertToBool(EveSettings.Get("NotifyWhenUnder5Mins"));
            NotifyWhenUnder1Hour = ConvertToBool(EveSettings.Get("NotifyWhenUnder1Hour"));
            CheckInterval = ConvertToInt(EveSettings.Get("CheckInterval"));
            ShowStatusOnTrayClick = ConvertToBool(EveSettings.Get("ShowStatusOnTrayClick"));
            StartWithWindows = ConvertToBool(EveSettings.Get("StartWithWindows"));
            ForceUpdateOnStart = ConvertToBool(EveSettings.Get("ForceUpdateOnStart"));
            api.SetKey(EveUserID, EveApiKey);

            mainForm.LoadSettings(
                EveUserID, EveApiKey, NotifyWhenNoSkill,
                NotifyWhenFinishedSkill, NotifyWhenUnder5Mins,
                NotifyWhenUnder1Hour, CheckInterval, ShowStatusOnTrayClick,
                StartWithWindows, ForceUpdateOnStart);
        }
        public void SaveSettings(
            string inEveUserID, string inEveApiKey, bool inNotifyWhenNoSkill,
            bool inNotifyWhenFinishedSkill, bool inNotifyWhenUnder5Mins,
            bool inNotifyWhenUnder1Hour, int inCheckInterval, bool inShowStatusOnTrayClick,
            bool inStartWithWindows, bool inForceUpdateOnStart)
        {
            EveUserID = inEveUserID;
            EveApiKey = inEveApiKey;
            NotifyWhenNoSkill = inNotifyWhenNoSkill;
            NotifyWhenFinishedSkill = inNotifyWhenFinishedSkill;
            NotifyWhenUnder5Mins = inNotifyWhenUnder5Mins;
            NotifyWhenUnder1Hour = inNotifyWhenUnder1Hour;
            CheckInterval = inCheckInterval;
            ShowStatusOnTrayClick = inShowStatusOnTrayClick;
            StartWithWindows = inStartWithWindows;
            ForceUpdateOnStart = inForceUpdateOnStart;
            EveSettings.Set("UserID", EveUserID);
            EveSettings.Set("ApiKey", EveApiKey);
            EveSettings.Set("NotifyWhenNoSkill", ConvertToString(NotifyWhenNoSkill));
            EveSettings.Set("NotifyWhenFinishedSkill", ConvertToString(NotifyWhenFinishedSkill));
            EveSettings.Set("NotifyWhenUnder5Mins", ConvertToString(NotifyWhenUnder5Mins));
            EveSettings.Set("NotifyWhenUnder1Hour", ConvertToString(NotifyWhenUnder1Hour));
            EveSettings.Set("NotifyWhenUnder1Hour", ConvertToString(NotifyWhenUnder1Hour));
            EveSettings.Set("CheckInterval", CheckInterval.ToString());
            EveSettings.Set("ShowStatusOnTrayClick", ConvertToString(ShowStatusOnTrayClick));
            EveSettings.Set("StartWithWindows", ConvertToString(StartWithWindows));
            EveSettings.Set("ForceUpdateOnStart", ConvertToString(ForceUpdateOnStart));
            EveSettings.Save();
            StartupWithWindows(StartWithWindows);
            api.SetKey(EveUserID, EveApiKey);
        }
        private int ConvertToInt(string value)
        {
            int output = 0;
            bool result = int.TryParse(value, out output);
            return output;
        }
        private bool ConvertToBool(string value)
        {
            if (value == "true")
                return true;
            else if (value == "false")
                return false;
            else
                return false;
        }
        private string ConvertToString(bool value)
        {
            if (value)
                return "true";
            else
                return "false";
        }
        
        private bool ReloadChars(bool showErrors)
        {
            if (EveApiKey == "" || EveUserID == "")
            {
                if (showErrors)
                    MessageBox.Show("Api key/User ID not set", "Cannot Update", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            ShowBusyIcon();
            bool result = false;
            if (CharsLoaded == false || api.characters.Count == 0)
            {
                result = api.LoadCharacters();
                if (result == false)
                {
                    if (showErrors)
                        MessageBox.Show(api.ErrorText, "Error Occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ShowIdleIcon();
                    return false;
                }
                else
                    CharsLoaded = true;
            }
            if (CharsLoaded)
            {
                result = api.LoadSkills();
                if (result == false)
                {
                    if (showErrors)
                        MessageBox.Show(api.ErrorText, "Error Occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ShowIdleIcon();
                    return false;
                }
            }
            LastCheck = DateTime.Now;
            EveApi.characterStatus cs = api.TrainingStatus();
            if (cs != null && cs.IsTraining && cs.SkillID != CurrentSkill)
            {
                //Reset notifications
                Notified1HourWarning = false;
                Notified5minWarning = false;
                LastNoSkillNotify = DateTime.MinValue;
                NotifiedSkillFinished = false;
                CurrentSkill = cs.SkillID;
            }
            if (api.CachedUntil != DateTime.MinValue)
                mainForm.UpdateCacheTime(api.CachedUntil);
            ShowIdleIcon();
            return true;
        }
        private void Settings_Click(object sender, System.EventArgs e)
        {
            mainForm.Show();
        }
        private void about_Click(object sender, System.EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }
        private void ForceReload_Click(object sender, System.EventArgs e)
        {
            ForceUpdate();
        }
        private void ShowInfo_Click(object sender, System.EventArgs e)
        {
            ShowSkillInfo();
        }
        private void trayicon_DoubleClick(object sender, System.EventArgs e)
        {
            if (ShowStatusOnTrayClick)
                ShowSkillInfo();
        }
        private void Exit_Click(object sender, System.EventArgs e)
        {
            Exit();
        }
        public void Exit()
        {
            EveSettings.Save();
            trayicon.Dispose();
            Application.Exit();
        }
        private int SecondsUntilCheck = 0;
        private bool WaitForRecheck = false;
        private void notifyTimer_Tick(object sender, EventArgs e)
        {
            if (WaitForRecheck)
            {
                if (SecondsUntilCheck <= 0)
                {
                    bool result = ReloadChars(false);
                    if (result)
                        WaitForRecheck = false;
                    else
                        SecondsUntilCheck = 20;
                }
                SecondsUntilCheck--;
            }
            DateTime checkTime = DateTime.Now.Subtract(new TimeSpan(CheckInterval, 0, 0));
            if (DateTime.Compare(checkTime, LastCheck) > 0 && !WaitForRecheck)
            {
                bool result = ReloadChars(false);
                if (result == false)
                {
                    if (NotifiedUpdateError == false)
                    {
                        NotifyWindow nw = new NotifyWindow();
                        nw.Text = "Cound not update Skill Information (" + api.ErrorText + ")";
                        nw.Title = "Update Error";
                        nw.WaitTime = 10000;
                        nw.Notify();
                        WaitForRecheck = true;
                        SecondsUntilCheck = 20;
                        NotifiedUpdateError = true;
                    }
                }
                else
                {
                    WaitForRecheck = false;
                    SecondsUntilCheck = 0;
                    NotifiedUpdateError = false;
                }
            }
            EveApi.characterStatus cs = api.TrainingStatus();

            //Update taskicon text
            trayicon.Text = "Eve Skill Notifier";
            if (cs == null)
                trayicon.Text += " - No Skill Info";
            else
            {
                trayicon.Text += " - Skill ETA : ";
                if (cs.SkillCompletionETA.TotalSeconds < 0)
                    trayicon.Text += "Training Complete";
                else
                {
                    string eta = "";
                    if (cs.SkillCompletionETA.Days > 0)
                        eta += cs.SkillCompletionETA.Days.ToString() + "d ";
                    if (cs.SkillCompletionETA.Hours > 0)
                        eta += cs.SkillCompletionETA.Hours.ToString() + "h ";
                    if (cs.SkillCompletionETA.Minutes > 0)
                        eta += cs.SkillCompletionETA.Minutes.ToString() + "m ";
                    if (cs.SkillCompletionETA.Seconds > 0)
                        eta += cs.SkillCompletionETA.Seconds.ToString() + "s";
                    trayicon.Text += eta;
                }
            }

            //Possible bug fix for no icon on windows start
            if (trayicon.Visible == false)
                trayicon.Visible = true;

            //Update settings window skillinfo
            string skillInfo = "";
            if (cs == null)
                skillInfo = "Unknown";
            else if (cs.IsTraining == false)
                skillInfo = "No Skill in Training";
            else
            {
                skillInfo = cs.CharName + " Currently Training \"" + cs.SkillName + "\" to Level " + cs.TrainingToLevel.ToString() + "\r\n";
                if (cs.SkillCompletionETA.TotalSeconds < 0)
                    skillInfo += "Training Complete";
                else
                {
                    string eta = "";
                    if (cs.SkillCompletionETA.Days > 0)
                        eta += cs.SkillCompletionETA.Days.ToString() + "d ";
                    if (cs.SkillCompletionETA.Hours > 0)
                        eta += cs.SkillCompletionETA.Hours.ToString() + "h ";
                    if (cs.SkillCompletionETA.Minutes > 0)
                        eta += cs.SkillCompletionETA.Minutes.ToString() + "m ";
                    if (cs.SkillCompletionETA.Seconds > 0)
                        eta += cs.SkillCompletionETA.Seconds.ToString() + "s";
                    skillInfo += "Skill ETA : " + eta;
                }
            }
            mainForm.UpdateSkillInfo(skillInfo);

            if (NotifyWhenNoSkill)
            {
                if (cs == null && CharsLoaded == true && DateTime.Compare(DateTime.Now.Subtract(new TimeSpan(0, 5, 0)), LastNoSkillNotify) >= 0)
                {
                    NotifyWindow nw = new NotifyWindow("Eve Skill Notifier", "No Skill in training");
                    nw.WaitTime = 10000;
                    nw.Notify();
                    LastNoSkillNotify = DateTime.Now;
                }
            }
            //Skill Finished notify
            if (   cs != null 
                && cs.SkillCompletionETA.TotalSeconds < 0 
                && !NotifiedSkillFinished 
                && NotifyWhenFinishedSkill)
            {
                NotifyWindow nw = new NotifyWindow(cs.CharName, 
                    "Training of Skill \n" + cs.SkillName + "\n has been completed");
                nw.WaitTime = 15000;
                nw.Notify();
                NotifiedSkillFinished = true;
            }
            //5 Mins notify
            if (   cs != null
                && cs.SkillCompletionETA.TotalMinutes < 5
                && Notified5minWarning == false
                && NotifyWhenUnder5Mins
                && NotifiedSkillFinished == false) // only show 1 notice
            {
                NotifyWindow nw = new NotifyWindow(cs.CharName, 
                    "Training of Skill \n" + cs.SkillName + "\n will complete in under 5 minutes");
                nw.WaitTime = 8000;
                nw.Notify();
                Notified5minWarning = true;
            }
            //1 Hour Notify
            if (   cs != null
                && cs.SkillCompletionETA.TotalHours < 1
                && Notified1HourWarning == false 
                && NotifyWhenUnder1Hour
                && Notified5minWarning == false     //
                && NotifiedSkillFinished == false)  // only show 1 notice
            {
                NotifyWindow nw = new NotifyWindow(cs.CharName, 
                    "Training of Skill \n" + cs.SkillName + "\n will complete in under 1 hour");
                nw.WaitTime = 8000;
                nw.Notify();
                Notified1HourWarning = true;
            }
        }

        public void ShowBusyIcon()
        {
            trayicon.Icon = new Icon(GetType(), "traybusy.ico");
        }
        public void ShowIdleIcon()
        {
            trayicon.Icon = new Icon(GetType(), "trayicon.ico");
        }
        public bool ForceUpdate()
        {
            return ReloadChars(true);
        }
        public void ShowSkillInfo()
        {
            if (!CharsLoaded)
            {
                NotifyWindow nw = new NotifyWindow("Eve Skill Notifier", 
                    "Character information not loaded / No Characters");
                nw.WaitTime = 5000;
                nw.Notify();
                return;
            }
            EveApi.characterStatus cs = api.TrainingStatus();
            if (cs == null)
            {
                NotifyWindow nw = new NotifyWindow("Eve Skill Notifier", "No Skill in training");
                nw.WaitTime = 5000;
                nw.Notify();
            }
            else if (cs != null && cs.SkillCompletionETA.TotalSeconds < 0)
            {
                NotifyWindow nw = new NotifyWindow(cs.CharName, 
                    "Training of Skill \"" + cs.SkillName + "\" has been completed");
                nw.WaitTime = 15000;
                nw.Notify();
            }
            else
            {
                NotifyWindow nw = new NotifyWindow();
                nw.Title = cs.CharName;
                nw.Text = "Training Skill\n"+cs.SkillName;
                if (cs.TrainingToLevel <= 5)
                    nw.Text += "\nTo Level "+cs.TrainingToLevel;
                int days = cs.SkillCompletionETA.Days;
                int hours = cs.SkillCompletionETA.Hours;
                int minutes = cs.SkillCompletionETA.Minutes;
                int seconds = cs.SkillCompletionETA.Seconds;
                nw.Text += "\nETA ";
                if (days > 0)
                {
                    nw.Text += days.ToString();
                    if (days == 1)
                        nw.Text += " Day, ";
                    else
                        nw.Text += " Days, ";
                }
                if (hours > 0)
                {
                    nw.Text += hours.ToString();
                    if (hours == 1)
                        nw.Text += " Hour, ";
                    else
                        nw.Text += " Hours, ";
                }
                if (minutes > 0)
                {
                    nw.Text += minutes.ToString();
                    if (minutes == 1)
                        nw.Text += " Minute, ";
                    else
                        nw.Text += " Minutes, ";
                }
                if (seconds > 0)
                {
                    nw.Text += seconds.ToString();
                    if (seconds == 1)
                        nw.Text += " Second ";
                    else
                        nw.Text += " Seconds ";
                }
                if (cs.SkillCompletionETA.TotalSeconds <= 0)
                    nw.Text += "Completion Imminent";
                nw.ActualWidth = 200;
                nw.Notify();
            }
        }
        private void StartupWithWindows(bool value)
        {
            RegistryKey regkey = Registry.CurrentUser.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Run", true);
            string runCommand = "\""+Environment.CurrentDirectory+@"\"+"EveSkillNotify.exe\" /quiet /dir:\""+Environment.CurrentDirectory+"\"";
            if (value)
            {
                string[] runValues = regkey.GetValueNames();
                bool regValueExists = false;
                foreach (string runValue in runValues)
                    if (runValue == "EveSkillNotify")
                        regValueExists = true;
                if (regValueExists) //confirm value
                {
                    string currentCommand = (string)regkey.GetValue("EveSkillNotify");
                    if (currentCommand != runCommand)
                        regkey.SetValue("EveSkillNotify", runCommand);
                }
                else
                    regkey.SetValue("EveSkillNotify", runCommand);
            }
            else
            {
                regkey.DeleteValue("EveSkillNotify", false);
            }
            Registry.CurrentUser.Flush();
        }
    }
}
